/*
 * This software is licensed under terms that can be found in the LICENSE file
 * in the root directory of this software component.
 *
 * Copyright (c) 2023, Georgy Moshkin <georgy.moshkin@outlook.com>
 * All rights reserved.
 *
 */

#include "uart.h"
#include "windows.h"

COM_TPort* myPortPtr;
COM_TPort myPort[COM_MAX_NUM];

void COM_Select(uint8_t n)
{
    myPortPtr=&myPort[n];
}

bool COM_Init(uint8_t n,
        uint8_t comNum,
        DWORD bps,
		uint16_t timeOut
		)
{
    COM_Select(n);
    myPortPtr->huartHandle=INVALID_HANDLE_VALUE;
    return openPort(&myPortPtr->huartHandle, comNum, bps);
}

bool COM_DeInit(uint8_t n)
{
    if (myPort[n].huartHandle!=INVALID_HANDLE_VALUE)
    {
        bool result=CloseHandle(myPort[n].huartHandle);
        myPort[n].huartHandle=INVALID_HANDLE_VALUE;
        return result;
    } else return false;
}

HANDLE COM_GetHandle(uint8_t n)
{
    return myPortPtr->huartHandle;
}

bool COM_Write(void *pData, uint16_t Size)
{
    return sendGet(&myPortPtr->huartHandle,(uint8_t*) pData,Size,0);
}

bool COM_Read(void *pData, uint16_t Size)
{
    return sendGet(&myPortPtr->huartHandle,(uint8_t*) pData,0,Size);
}

bool sendGet(HANDLE *hPort, uint8_t *data, DWORD sendLen, DWORD needGetLen)
{

    if (*hPort!=INVALID_HANDLE_VALUE)
    {
        DWORD sent;
        DWORD bytesTrans;


        if (!WriteFile(*hPort,
                       data,
                       sendLen,
                       &sent,
                       NULL))
        {
            return false;
        }

        if (needGetLen>0)
        {


            memset(data, (char) 0, needGetLen);

            if (!ReadFile(*hPort,
                          data,
                          needGetLen,
                          &bytesTrans,
                          NULL))
            {

                return false;

            }


            if (bytesTrans!=needGetLen)
            {
                return false;
            }


            return true;
        }

    }

    return false;
}


bool openPort(HANDLE *hPort, uint8_t pnum, DWORD bps)
{

    wchar_t comStr[64];
    wsprintf(comStr,L"\\\\.\\COM%u",pnum);

    *hPort = CreateFile(comStr, GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);

    if (*hPort!=INVALID_HANDLE_VALUE)
    {

        _COMMTIMEOUTS comTimeouts;

        comTimeouts.ReadIntervalTimeout = 0;
        comTimeouts.ReadTotalTimeoutMultiplier = 2; // bytes x 2ms
        comTimeouts.ReadTotalTimeoutConstant = 100; // 100ms
        comTimeouts.WriteTotalTimeoutMultiplier = 0;
        comTimeouts.WriteTotalTimeoutConstant = 0;

        SetCommTimeouts(*hPort,&comTimeouts);

        DCB dcb = {0};
        dcb.DCBlength = sizeof(dcb);

        dcb.BaudRate = bps;
        dcb.ByteSize = 8;
        dcb.Parity   = NOPARITY;
        dcb.StopBits = ONESTOPBIT;
        dcb.fBinary  = 1;

        dcb.fOutxCtsFlow = FALSE;
        dcb.fOutxDsrFlow = FALSE;
        dcb.fOutX        = FALSE;
        dcb.fInX = FALSE;

        // Fix for HDD software virtual serial port:
        dcb.XonChar      = 0x11;
        dcb.XoffChar     = 0x13;
        dcb.XoffLim      = 100;
        dcb.XonLim       = 100;

        if (SetCommState(*hPort, &dcb))
        {
            return true;
        }
        else
        {
            return false;
        }

        //TODO: PurgeComm(*hPort, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);
    }
    else return false;



}


